import 'dart:async';

import 'package:flutter/material.dart';

class DelayedWidgetBuilder extends StatefulWidget {
  const DelayedWidgetBuilder(
      {Key? key, required this.child, required this.delay})
      : super(key: key);

  static const Key progressIndicatorKey = Key('progressIndicatorKey');

  final Widget child;

  final Duration delay;

  @override
  State<StatefulWidget> createState() => DelayedWidgetBuilderState();
}

class DelayedWidgetBuilderState extends State<DelayedWidgetBuilder> {
  bool isReady = false;
  late Timer timer;

  @override
  void initState() {
    super.initState();
    timer = Timer(widget.delay, () {
      setState(() {
        isReady = true;
      });
    });
  }

  @override
  void dispose() {
    super.dispose();
    timer.cancel();
  }

  @override
  Widget build(BuildContext context) {
    if (isReady) {
      return widget.child;
    } else {
      return const CircularProgressIndicator(
        key: DelayedWidgetBuilder.progressIndicatorKey,
      );
    }
  }
}
